@@ -1,9 +1,9 @@
 # Copyright (C) 2007, One Laptop Per Child
 # Copyright (C) 2009,14 Walter Bender
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from gettext import gettext as _
 from gettext import ngettext
@@ -23,6 +22,7 @@ import os
 import time
 
 from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gdk
 
@@ -34,7 +34,7 @@ from sugar3.graphics.palette import ToolInvoker
 from sugar3.graphics.palettemenu import PaletteMenuBox
 from sugar3.graphics.palettemenu import PaletteMenuItem
 from sugar3.graphics.palettemenu import PaletteMenuItemSeparator
-from sugar3.graphics.icon import Icon
+from sugar3.graphics.icon import Icon, EventIcon
 from sugar3.graphics.alert import Alert
 from sugar3.graphics.xocolor import XoColor
 from sugar3.graphics import iconentry
@@ -85,6 +85,7 @@ class MainToolbox(ToolbarBox):
         self._filter_type = default_filter_type
         self._what_filter = default_what_filter
         self._when_filter = None
+
         self._default_what_filter = default_what_filter
         self._default_filter_type = default_filter_type
 
@@ -106,13 +107,21 @@ class MainToolbox(ToolbarBox):
         self.toolbar.insert(self._favorite_button, -1)
         self._favorite_button.show()
 
-        self._what_widget_contents = None
-        self._what_widget = Gtk.ToolItem()
-        self._what_search_button = FilterToolItem(
-            'view-type', _('Anything'), self._what_widget)
-        self._what_widget.show()
-        self.toolbar.insert(self._what_search_button, -1)
-        self._what_search_button.show()
+        self._proj_list_button = ToggleToolButton('project-box')
+        self._proj_list_button.set_tooltip(_('Projects'))
+        self._proj_list_button.connect('toggled',
+                                       self._proj_list_button_clicked_cb)
+        self.toolbar.insert(self._proj_list_button, -1)
+        self._proj_list_button.show()
+
+        if not self._proj_list_button.props.active:
+            self._what_widget_contents = None
+            self._what_widget = Gtk.ToolItem()
+            self._what_search_button = FilterToolItem(
+                'view-type', _('Anything'), self._what_widget)
+            self._what_widget.show()
+            self.toolbar.insert(self._what_search_button, -1)
+            self._what_search_button.show()
 
         self._when_search_button = FilterToolItem(
             'view-created', _('Anytime'), self._get_when_search_items())
@@ -140,10 +149,10 @@ class MainToolbox(ToolbarBox):
 
         self.refresh_filters()
 
-        self.connect('size_allocate', self.__size_allocate_cb)
+        self.connect('size-allocate', self.__size_allocate_cb)
 
     def __size_allocate_cb(self, widget, allocation):
-        GObject.idle_add(self._update_buttons, allocation.width)
+        GLib.idle_add(self._update_buttons, allocation.width)
 
     def _update_buttons(self, toolbar_width):
         # Show the label next to the button icon if there is room on
@@ -231,7 +240,10 @@ class MainToolbox(ToolbarBox):
         if self._favorite_button.props.active:
             query['keep'] = 1
 
-        if self._what_filter:
+        if self._proj_list_button.props.active:
+            query['activity'] = 'org.sugarlabs.Project'
+
+        elif self._what_filter:
             filter_type = self._filter_type
             value = self._what_filter
 
@@ -305,7 +317,7 @@ class MainToolbox(ToolbarBox):
 
     def _search_entry_activated_cb(self, search_entry):
         if self._autosearch_timer:
-            GObject.source_remove(self._autosearch_timer)
+            GLib.source_remove(self._autosearch_timer)
         self._update_if_needed()
 
     def _search_entry_changed_cb(self, search_entry):
@@ -314,9 +326,9 @@ class MainToolbox(ToolbarBox):
             return
 
         if self._autosearch_timer:
-            GObject.source_remove(self._autosearch_timer)
-        self._autosearch_timer = GObject.timeout_add(_AUTOSEARCH_TIMEOUT,
-                                                     self._autosearch_timer_cb)
+            GLib.source_remove(self._autosearch_timer)
+        self._autosearch_timer = GLib.timeout_add(_AUTOSEARCH_TIMEOUT,
+                                                  self._autosearch_timer_cb)
 
     def _autosearch_timer_cb(self):
         logging.debug('_autosearch_timer_cb')
@@ -433,7 +445,7 @@ class MainToolbox(ToolbarBox):
                              'file': activity_info.get_icon(),
                              'callback': self._what_palette_cb,
                              'id': bundle_id})
-                    except GObject.GError, exception:
+                    except GLib.GError as exception:
                         # fall back to generic icon
                         logging.warning('Falling back to default icon for'
                                         ' "what" filter because %r (%r) has an'
@@ -446,13 +458,7 @@ class MainToolbox(ToolbarBox):
                              'callback': self._what_palette_cb,
                              'id': bundle_id})
         finally:
-            def _cmp(a, b):
-                if a['label'] < b['label']:
-                    return -1
-                else:
-                    return 1
-
-            for item in sorted(what_list_activities, _cmp):
+            for item in sorted(what_list_activities, key=lambda x: x['label']):
                 self._what_list.append(item)
 
             if self._what_widget_contents is not None:
@@ -461,6 +467,15 @@ class MainToolbox(ToolbarBox):
             self._what_widget.add(self._what_widget_contents)
             self._what_widget_contents.show()
 
+    def _proj_list_button_clicked_cb(self, proj_list_button):
+        if self._proj_list_button.props.active:
+            self._what_widget.hide()
+            self._what_search_button.hide()
+        else:
+            self._what_widget.show()
+            self._what_search_button.show()
+        self._update_if_needed()
+
     def __favorite_button_toggled_cb(self, favorite_button):
         self._update_if_needed()
 
@@ -491,6 +506,11 @@ class MainToolbox(ToolbarBox):
 
         self._favorite_button.props.active = False
 
+        if self._proj_list_button.props.active:
+            self._what_widget.show()
+            self._what_search_button.show()
+            self._proj_list_button.props.active = False
+
         self._update_if_needed()
 
 
@@ -563,15 +583,15 @@ class DetailToolbox(ToolbarBox):
                     alert_window=journalwindow.get_journal_window())
 
     def _copy_clicked_cb(self, button):
-        button.palette.popup(immediate=True, state=Palette.SECONDARY)
+        button.palette.popup(immediate=True)
 
     def _refresh_clicked_cb(self, button):
-        button.palette.popup(immediate=True, state=Palette.SECONDARY)
+        button.palette.popup(immediate=True)
 
     def _duplicate_clicked_cb(self, button):
         try:
             model.copy(self._metadata, '/')
-        except IOError, e:
+        except IOError as e:
             logging.exception('Error while copying the entry.')
             self.emit('volume-error',
                       _('Error while copying the entry. %s') % (e.strerror, ),
@@ -727,9 +747,15 @@ class SortingButton(ToolButton):
             menu_box.append_item(button)
 
     def __sort_type_changed_cb(self, widget, property_, icon_name):
+        if self._property == property_:
+            if self._order == Gtk.SortType.ASCENDING:
+                self._order = Gtk.SortType.DESCENDING
+            else:
+                self._order = Gtk.SortType.ASCENDING
+        else:
+            self._order = Gtk.SortType.ASCENDING
+
         self._property = property_
-        # FIXME: Implement sorting order
-        self._order = Gtk.SortType.ASCENDING
         self.emit('sort-property-changed')
 
         self.props.icon_name = icon_name
@@ -762,7 +788,7 @@ class EditToolbox(ToolbarBox):
 
     def display_selected_entries_status(self):
         info_widget = self._multi_select_info_widget
-        GObject.idle_add(info_widget.display_selected_entries)
+        GLib.idle_add(info_widget.display_selected_entries)
 
     def set_total_number_of_entries(self, total):
         self._multi_select_info_widget.set_total_number_of_entries(total)
@@ -847,7 +873,7 @@ class BatchCopyButton(ToolButton):
 
     def __clicked_cb(self, button):
         self._refresh_menu_options()
-        button.palette.popup(immediate=True, state=Palette.SECONDARY)
+        button.palette.popup(immediate=True)
 
     def __get_uid_list_cb(self):
         model = self._journalactivity.get_list_view().get_model()
@@ -922,11 +948,11 @@ class FilterToolItem(Gtk.ToolButton):
     def set_widget_icon(self, icon_name=None, file_name=None):
         if file_name is not None:
             icon = Icon(file=file_name,
-                        icon_size=style.SMALL_ICON_SIZE,
+                        pixel_size=style.SMALL_ICON_SIZE,
                         xo_color=XoColor('white'))
         else:
             icon = Icon(icon_name=icon_name,
-                        icon_size=style.SMALL_ICON_SIZE,
+                        pixel_size=style.SMALL_ICON_SIZE,
                         xo_color=XoColor('white'))
         self.set_icon_widget(icon)
         icon.show()
@@ -940,10 +966,6 @@ class FilterToolItem(Gtk.ToolButton):
         self._label_widget.set_markup(label)
         self._label = label
 
-    def __destroy_cb(self, icon):
-        if self._palette_invoker is not None:
-            self._palette_invoker.detach()
-
     def create_palette(self):
         return None
 
@@ -953,7 +975,7 @@ class FilterToolItem(Gtk.ToolButton):
     def set_palette(self, palette):
         self._palette_invoker.palette = palette
 
-    palette = GObject.property(
+    palette = GObject.Property(
         type=object, setter=set_palette, getter=get_palette)
 
     def get_palette_invoker(self):
@@ -963,7 +985,7 @@ class FilterToolItem(Gtk.ToolButton):
         self._palette_invoker.detach()
         self._palette_invoker = palette_invoker
 
-    palette_invoker = GObject.property(
+    palette_invoker = GObject.Property(
         type=object, setter=set_palette_invoker, getter=get_palette_invoker)
 
     def do_draw(self, cr):
@@ -983,6 +1005,10 @@ class FilterToolItem(Gtk.ToolButton):
         return False
 
 
+if hasattr(FilterToolItem, 'set_css_name'):
+    FilterToolItem.set_css_name('filtertoolbutton')
+
+
 def set_palette_list(palette_list):
     if 'icon' in palette_list[0]:
         _menu_item = PaletteMenuItem(icon_name=palette_list[0]['icon'],
@@ -1053,5 +1079,48 @@ def set_palette_list(palette_list):
                                          (ny + 1) * menuitem_height)
         scrolled_window.add_with_viewport(grid)
         return scrolled_window
-    else:
-        return grid
+    return grid
+
+
+class AddNewBar(Gtk.Box):
+
+    activate = GObject.Signal('activate', arg_types=[str])
+
+    def __init__(self, placeholder=None):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
+
+        self._button = EventIcon(icon_name='list-add')
+        self._button.connect('button-release-event',
+                             self.__button_release_event_cb)
+        self._button.fill_color = style.COLOR_TOOLBAR_GREY.get_svg()
+        self._button.set_tooltip(_('Add New'))
+        self.pack_start(self._button, False, True, 0)
+        self._button.show()
+
+        self._entry = iconentry.IconEntry()
+        self._entry.connect('key-press-event', self.__key_press_cb)
+        if placeholder is None:
+            placeholder = _('Add new entry')
+        self._entry.set_placeholder_text(placeholder)
+        self._entry.add_clear_button()
+        self.pack_start(self._entry, True, True, 0)
+        self._entry.show()
+
+    def get_entry(self):
+        return self._entry
+
+    def get_button(self):
+        return self._button
+
+    def __key_press_cb(self, window, event):
+        if event.keyval == Gdk.KEY_Return:
+            return self._maybe_activate()
+
+    def __button_release_event_cb(self, button, event):
+        self._maybe_activate()
+
+    def _maybe_activate(self):
+        if self._entry.props.text:
+            self.activate.emit(self._entry.props.text)
+            self._entry.props.text = ''
+            return True
