@@ -51,7 +51,7 @@ class TreeView(Gtk.TreeView):
         'volume-error': (GObject.SignalFlags.RUN_FIRST, None,
                          ([str, str])),
         'choose-project': (GObject.SignalFlags.RUN_FIRST, None,
-                          ([object])),
+                           ([object])),
     }
 
     def __init__(self, journalactivity):
@@ -244,8 +244,7 @@ class BaseListView(Gtk.Bin):
             return None
         if self._query['mountpoints'] == ['/']:
             return not object_id.startswith('/')
-        else:
-            return object_id.startswith(self._query['mountpoints'][0])
+        return object_id.startswith(self._query['mountpoints'][0])
 
     def _add_columns(self):
         if self._enable_multi_operations:
@@ -415,6 +414,11 @@ class BaseListView(Gtk.Bin):
         self.tree_view.queue_draw_area(cell_rect.x, cell_rect.y,
                                        cell_rect.width, cell_rect.height)
 
+        # HACK for https://bugs.sugarlabs.org/ticket/4944
+        # Icon does not update automatically if there is only one journal entry
+        if len(self._model.get_all_ids()) == 1:
+            self._do_refresh()
+
     def __select_set_data_cb(self, column, cell, tree_model, tree_iter,
                              data):
         uid = tree_model[tree_iter][ListModel.COLUMN_UID]
@@ -450,7 +454,8 @@ class BaseListView(Gtk.Bin):
         window = self.get_toplevel().get_window()
         if window is not None:
             window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
-        GObject.idle_add(self._do_refresh, new_query)
+            Gdk.flush()
+        GLib.idle_add(self._do_refresh, new_query)
 
     def _do_refresh(self, new_query=False):
         if self._model is not None:
@@ -468,6 +473,7 @@ class BaseListView(Gtk.Bin):
         window = self.get_toplevel().get_window()
         if window is not None:
             window.set_cursor(None)
+            Gdk.flush()
 
     def __model_ready_cb(self, tree_model):
         self._stop_progress_bar()
@@ -515,7 +521,8 @@ class BaseListView(Gtk.Bin):
                     if self.get_projects_view_active():
                         show_message_text = _('No Projects')
 
-                    self._show_message(show_message_text,
+                    self._show_message(
+                        show_message_text,
                         show_clear_query=self._can_clear_query())
         else:
             self._clear_message()
@@ -642,9 +649,8 @@ class BaseListView(Gtk.Bin):
                                            cel_rect.height)
             if path == end_path:
                 break
-            else:
-                next_iter = tree_model.iter_next(tree_model.get_iter(path))
-                path = tree_model.get_path(next_iter)
+            next_iter = tree_model.iter_next(tree_model.get_iter(path))
+            path = tree_model.get_path(next_iter)
 
     def _set_dirty(self):
         if self._fully_obscured or self._updates_disabled:
@@ -672,13 +678,13 @@ class BaseListView(Gtk.Bin):
             if self._update_dates_timer is None:
                 logging.debug('Adding date updating timer')
                 self._update_dates_timer = \
-                    GObject.timeout_add_seconds(UPDATE_INTERVAL,
-                                                self.__update_dates_timer_cb)
+                    GLib.timeout_add_seconds(UPDATE_INTERVAL,
+                                             self.__update_dates_timer_cb)
         else:
             self._fully_obscured = True
             if self._update_dates_timer is not None:
                 logging.debug('Remove date updating timer')
-                GObject.source_remove(self._update_dates_timer)
+                GLib.source_remove(self._update_dates_timer)
                 self._update_dates_timer = None
 
     def __update_dates_timer_cb(self):
@@ -717,7 +723,7 @@ class ListView(BaseListView):
         'title-edit-finished': (GObject.SignalFlags.RUN_FIRST, None,
                                 ([])),
         'project-view-activate': (GObject.SignalFlags.RUN_FIRST, None,
-                                 ([object])),
+                                  ([object])),
     }
 
     def __init__(self, journalactivity, enable_multi_operations=False):
@@ -875,4 +881,4 @@ class CellRendererBuddy(CellRendererIcon):
             self.props.icon_name = 'computer-xo'
             self.props.xo_color = xo_color
 
-    buddy = GObject.property(type=object, setter=set_buddy)
+    buddy = GObject.Property(type=object, setter=set_buddy)
