@@ -20,6 +20,7 @@ import logging
 import os
 
 from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -56,7 +57,7 @@ class ObjectPalette(Palette):
         'volume-error': (GObject.SignalFlags.RUN_FIRST, None,
                          ([str, str])),
         'choose-project': (GObject.SignalFlags.RUN_FIRST, None,
-                          ([object])),
+                           ([object])),
     }
 
     def __init__(self, journalactivity, metadata, detail=False):
@@ -358,11 +359,6 @@ class CopyMenuBuilder():
         self._menu.remove(volume_menu)
         del self._volumes[mount.get_root().get_path()]
 
-    def __destroy_cb(self, widget):
-        volume_monitor = Gio.VolumeMonitor.get()
-        volume_monitor.disconnect(self._mount_added_hid)
-        volume_monitor.disconnect(self._mount_removed_hid)
-
 
 class VolumeMenu(MenuItem):
     __gtype_name__ = 'JournalVolumeMenu'
@@ -596,7 +592,7 @@ class BatchOperator(GObject.GObject):
 
     def _show_confirmation_alert(self):
         self._journalactivity.freeze_ui()
-        GObject.idle_add(self.__show_confirmation_alert_internal)
+        GLib.idle_add(self.__show_confirmation_alert_internal)
 
     def __show_confirmation_alert_internal(self):
         # Show a alert requesting confirmation before run the batch operation
@@ -626,9 +622,9 @@ class BatchOperator(GObject.GObject):
             # this is only in the case the operation already started
             # and the user want stop it.
             self._stop_batch_execution()
-        elif hasattr(self, '_object_index') == False:
+        elif not hasattr(self, '_object_index'):
             self._object_index = 0
-            GObject.idle_add(self._operate_by_uid_internal)
+            GLib.idle_add(self._operate_by_uid_internal)
 
     def _operate_by_uid_internal(self):
         # If there is still some uid left, proceed with the operation.
@@ -647,7 +643,7 @@ class BatchOperator(GObject.GObject):
                 'object_title': title}
 
             self._confirmation_alert.props.msg = alert_message
-            GObject.idle_add(self._operate_per_metadata, metadata)
+            GLib.idle_add(self._operate_per_metadata, metadata)
         else:
             self._finish_batch_execution()
 
@@ -656,7 +652,7 @@ class BatchOperator(GObject.GObject):
 
         # process the next
         self._object_index = self._object_index + 1
-        GObject.idle_add(self._operate_by_uid_internal)
+        GLib.idle_add(self._operate_by_uid_internal)
 
     def _stop_batch_execution(self):
         self._object_index = len(self._uid_list)
